﻿using System;

// Product
class Pizza
{
    private string dough;
    private string sauce;
    private string topping;

    public void SetDough(string dough)
    {
        this.dough = dough;
    }

    public void SetSauce(string sauce)
    {
        this.sauce = sauce;
    }

    public void SetTopping(string topping)
    {
        this.topping = topping;
    }

    public void Open()
    {
        Console.WriteLine("Pizza with {0} dough, {1} sauce and {2} topping. Mmm.", dough, sauce, topping);
    }
}

// Abstract Builder
abstract class PizzaBuilder
{
    protected Pizza pizza;

    public Pizza GetPizza()
    {
        return pizza;
    }

    public void CreateNewPizzaProduct()
    {
        pizza = new Pizza();
    }

    public abstract void BuildDough();
    public abstract void BuildSauce();
    public abstract void BuildTopping();
}

// Concrete Builder 1
class HawaiianPizzaBuilder : PizzaBuilder
{
    public override void BuildDough()
    {
        pizza.SetDough("cross");
    }

    public override void BuildSauce()
    {
        pizza.SetSauce("mild");
    }

    public override void BuildTopping()
    {
        pizza.SetTopping("ham and pineapple");
    }
}

// Concrete Builder 2
class SpicyPizzaBuilder : PizzaBuilder
{
    public override void BuildDough()
    {
        pizza.SetDough("pan baked");
    }

    public override void BuildSauce()
    {
        pizza.SetSauce("hot");
    }

    public override void BuildTopping()
    {
        pizza.SetTopping("pepperoni and salami");
    }
}

// Director
class Cook
{
    private PizzaBuilder pizzaBuilder;

    public void SetPizzaBuilder(PizzaBuilder pb)
    {
        pizzaBuilder = pb;
    }

    public void ConstructPizza()
    {
        pizzaBuilder.CreateNewPizzaProduct();
        pizzaBuilder.BuildDough();
        pizzaBuilder.BuildSauce();
        pizzaBuilder.BuildTopping();
    }

    public Pizza GetPizza()
    {
        return pizzaBuilder.GetPizza();
    }
}

// Client code
class Builder
{
    static void Main()
    {
        Cook cook = new Cook();
        PizzaBuilder hawaiianPizzaBuilder = new HawaiianPizzaBuilder();
        PizzaBuilder spicyPizzaBuilder = new SpicyPizzaBuilder();

        cook.SetPizzaBuilder(hawaiianPizzaBuilder);
        cook.ConstructPizza();

        Pizza hawaiianPizza = cook.GetPizza();
        hawaiianPizza.Open();

        cook.SetPizzaBuilder(spicyPizzaBuilder);
        cook.ConstructPizza();

        Pizza spicyPizza = cook.GetPizza();
        spicyPizza.Open();

        Console.ReadLine();
    }
}
